@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -212,7 +212,7 @@ static int dos_helper(void)
     show_ints(HI(bx), LO(bx));
     break;
 
-  case DOS_HELPER_PRINT_STRING:	/* SHOW INTS, BH-BL */
+  case DOS_HELPER_PRINT_STRING:	/* PRINT STRING ES:DX */
     g_printf("DOS likes us to print a string\n");
     ds_printf("DOS to EMU: \"%s\"\n",SEG_ADR((char *), es, dx));
     break;
@@ -419,6 +419,18 @@ static int dos_helper(void)
       break;
     }
 
+  case DOS_HELPER_ASPI_HELPER: {
+#ifdef ASPI_SUPPORT
+      extern void aspi_helper(int);
+      A_printf("ASPI: in 0x41 handler! ax=0x%04x, bx=0x%04x, dx=0x%04x, "
+           "cx=0x%04x\n", LWORD(eax), LWORD(ebx), LWORD(edx), LWORD(ecx));
+      aspi_helper(HI(ax));
+#else
+      LWORD(eax) = 0;
+#endif
+      break;
+  }
+
   case DOS_HELPER_RUN_UNIX:
     g_printf("Running Unix Command\n");
     run_unix_command(SEG_ADR((char *), es, dx));
@@ -437,16 +449,9 @@ static int dos_helper(void)
     break;   
 
   case DOS_HELPER_0x53:
-    if (config.secure) {
-      CARRY;
-      LWORD(eax) = 0;
-      break;
-    }
     {
-	PRIV_SAVE_AREA
-	enter_priv_off();
-	LWORD(eax) = system(SEG_ADR((char *), es, dx));
-	leave_priv_setting();
+        extern int run_system_command(char *);
+        LWORD(eax) = run_system_command(SEG_ADR((char *), es, dx));
 	break;
     }
 
@@ -1088,9 +1093,11 @@ static int ms_dos(int nr)
     return 0;
 
   default:
+#ifndef USE_NEW_INT
     if (!in_dpmi)
       g_printf("INT21 (0x%02x):  we shouldn't be here! ax=0x%04x, bx=0x%04x\n",
 	     nr, LWORD(eax), LWORD(ebx));
+#endif
     return 0;
   }
   return 1;
