@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 #include <stdio.h>
 #include <termios.h>
 #include <unistd.h>
@@ -23,9 +29,6 @@
 #include "dos2linux.h"
 #include "priv.h"
 #include "utilities.h"
-#ifdef X86_EMULATOR
-#include "cpu-emu.h"
-#endif
 #include "mhpdbg.h"
 
 
@@ -45,6 +48,9 @@ struct debug_flags d =
 int kernel_version_code = 0;
 int config_check_only = 0;
 
+int dosemu_argc;
+char **dosemu_argv;
+
 static void     check_for_env_autoexec_or_config(void);
 int     parse_debugflags(const char *s, unsigned char flag);
 static void     usage(void);
@@ -101,9 +107,6 @@ config_defaults(void)
     config.rdtsc = 0;
     config.mathco = 0;
     config.smp = 0;
-#ifdef X86_EMULATOR
-    config.emuspeed = 50;	/* instruction cycles per us */
-#endif
 
     open_proc_scan("/proc/cpuinfo");
     switch (get_proc_intvalue_by_key(
@@ -146,9 +149,6 @@ config_defaults(void)
       config.realcpu, config.mathco, config.rdtsc);
 
     config.hdiskboot = 1;	/* default hard disk boot */
-#ifdef X86_EMULATOR
-    config.cpuemu = 0;
-#endif
     config.mem_size = 640;
     config.ems_size = 0;
     config.ems_frame = 0xd000;
@@ -242,7 +242,6 @@ config_defaults(void)
     config.emusys = (char *) NULL;
     config.emubat = (char *) NULL;
     config.emuini = (char *) NULL;
-    tmpdir = strdup(tempnam("/tmp", "dosemu"));
     config.dosbanner = 1;
     config.allowvideoportaccess = 0;
 
@@ -282,6 +281,8 @@ config_defaults(void)
     config.mpu401_base = 0x330;
 
     config.vnet = 1;
+
+    memset(config.features, 0, sizeof(config.features));
 }
 
 void dump_config_status(void)
@@ -312,9 +313,6 @@ void dump_config_status(void)
 
     fprintf(out, "pci %d\nrdtsc %d\nmathco %d\nsmp %d\n",
                  config.pci, config.rdtsc, config.mathco, config.smp);
-#ifdef X86_EMULATOR
-    fprintf(out, "emuspeed %d\ncpuemu %d\n", config.emuspeed, config.cpuemu);
-#endif
 
     fprintf(out, "hdiskboot %d\nmem_size %d\n",
         config.hdiskboot, config.mem_size);
@@ -336,6 +334,8 @@ void dump_config_status(void)
         config.term_esc_char, config.term_color, config.term_updatefreq);
     switch (config.term_charset) {
       case CHARSET_LATIN: s = "latin"; break;
+      case CHARSET_LATIN1: s = "latin1"; break;
+      case CHARSET_LATIN2: s = "latin2"; break;
       case CHARSET_IBM: s = "ibm"; break;
       case CHARSET_FULLIBM: s = "fullibm"; break;
       default: s = "(unknown)";
@@ -459,6 +459,14 @@ void dump_config_status(void)
 	}
     }
 
+    {
+	int i;
+	int size = sizeof(config.features) / sizeof(config.features[0]);
+	for (i = 0; i < size; i++) {
+	  fprintf(out, "feature_%d %d\n", i, config.features[i]);
+	}
+    }
+
     fprintf(out, "\nSOUND:\nsb_base 0x%x\nsb_dma %d\nsb_irq %d\nmpu401_base 0x%x\nsb_dsp \"%s\"\nsb_mixer \"%s\"\n",
         config.sb_base, config.sb_dma, config.sb_irq, config.mpu401_base, config.sb_dsp, config.sb_mixer);
     fprintf(out, "\n--------------end of runtime configuration dump -------------\n");
@@ -571,6 +579,9 @@ config_init(int argc, char **argv)
       exit(0);
     }
 
+    dosemu_argc = argc;
+    dosemu_argv = argv;
+
     memset(usedoptions,0,sizeof(usedoptions));
     memcheck_type_init();
     our_envs_init(0);
@@ -730,7 +741,7 @@ config_init(int argc, char **argv)
     optind = 0;
 #endif
     opterr = 0;
-    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:v:VNtT:sgh:H:x:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:v:VNtsgh:H:x:KLm23456e:dXY:Z:E:o:Ou:")) != EOF) {
 	/* Note: /etc/dosemu.conf may have disallowed some options
 	 *	 ( by removing them from $DOSEMU_OPTIONS ).
 	 *	 We skip them by re-checking 'usedoptions'.
@@ -834,11 +845,6 @@ config_init(int argc, char **argv)
 	    warn("DOS will not be started\n");
 	    config.exitearly = 1;
 	    break;
-	case 'T':
-	    g_printf("Using tmpdir=%s\n", optarg);
-	    free(tmpdir);
-	    tmpdir = strdup(optarg);
-	    break;
 	case 't':
 	    g_printf("doing timer emulation\n");
 	    config.timers = 1;
@@ -971,9 +977,9 @@ int parse_debugflags(const char *s, unsigned char flag)
     char            c;
     int ret = 0;
 #ifdef X_SUPPORT
-    const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrSe";
+    const char      allopts[] = "dARWDCvXkiTsm#pgcwhIExMnPrSe";
 #else
-    const char      allopts[] = "dRWDCvkiTsm#pgcwhIExMnPrSe";
+    const char      allopts[] = "dARWDCvkiTsm#pgcwhIExMnPrSe";
 #endif
 
     /*
@@ -1076,11 +1082,9 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'S':		/* SOUND */
 	    d.sound = flag;
 	    break;
-#ifdef X86_EMULATOR
-	case 'e':		/* cpu-emu */
-	    d.emu = flag;
+	case 'A':		/* ASPI */
+	    d.aspi = flag;
 	    break;
-#endif
 	case 'a':{		/* turn all on/off depending on flag */
 		char           *newopts = (char *) malloc(strlen(allopts) + 2);
 
@@ -1157,7 +1161,6 @@ usage(void)
 	"    -O write debugmessages to stderr\n"
 	"    -o FILE put debugmessages in file\n"
 	"    -P copy debugging output to FILE\n"
-	"    -T DIR set tmpdir\n"
 	"    -t try new timer code (#)\n"
 	"    -u set user configuration variable 'confvar' prefixed by 'u_'.\n"
 	"    -V use BIOS-VGA video modes (!#%%)\n"
