@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -19,6 +19,8 @@
 #include "keyb_clients.h"
 #include "keyboard.h"
 #include "utilities.h"
+#include "video.h"
+#include "dosemu_select.h"
 
 #ifndef VOID
 #  define VOID void
@@ -533,7 +535,7 @@ static Keymap_Scan_Type Dosemu_Ctrl_keys[] =
 
 /* This table is for translating the ESC key. */
 /* The entries with 0x00 are not going to be used by any sane person. */
-Bit8u Ctrl_Char_Scan_Codes[32] =
+static Bit8u Ctrl_Char_Scan_Codes[32] =
 {
   0x03,			/* ^@ */
   0x1E,			/* ^A */
@@ -1043,7 +1045,7 @@ static void slang_send_scancode(unsigned long lscan, unsigned char ch)
   }
 }
 
-void do_slang_getkeys(void)
+static void n_do_slang_getkeys(void)
 {
   SLang_Key_Type *key;
   unsigned long scan;
@@ -1317,7 +1319,7 @@ static void sl_print_error(char *str)
  * DANG_END_FUNCTION
  */
    
-int slang_keyb_init(void) {
+static int n_slang_keyb_init(void) {
   struct termios buf;
 
   k_printf("KBD: slang_keyb_init()\n");
@@ -1328,8 +1330,33 @@ int slang_keyb_init(void) {
   fcntl(kbd_fd, F_SETFL, O_RDONLY | O_NONBLOCK);
    
   if (tcgetattr(kbd_fd, &save_termios) < 0) {
-    error("slang_keyb_init(): Couldn't tcgetattr(kbd_fd,...) !\n");
-    leavedos(66);
+    int ignore = 0;
+    if (errno == EINVAL) {
+      if ( (config.cardtype == CARD_NONE)
+          || !strcmp(getenv("TERM"),"dumb")        /* most cron's have this */
+          || !strcmp(getenv("TERM"),"none")        /* ... some have this */
+          || !strcmp(getenv("TERM"),"dosemu-none") /* ... when called recursivly */
+                                                ) {
+        /*
+         * We assume we are running without a terminal (e.g. in a cronjob).
+         * Hence, we setup the TERM variable to "dosemu-none",
+         * set a suitable TERMCAP entry ... and ignore the rest
+         */
+        setenv("TERM", "dosemu-none", 1);
+        setenv("TERMCAP",
+          "dosemu-none|for running DOSEMU without real terminal:"
+          ":am::co#80:it#8:li#25:"
+          ":ce=\\E[K:cl=\\E[H\\E[2J:cm=\\E[%i%d;%dH:do=\\E[B:ho=\\E[H:"
+          ":le=\\E[D:nd=\\E[C:ta=^I:up=\\E[A:",
+          1
+        );
+        ignore = 1;
+      }
+    }
+    if (!ignore) {
+      error("slang_keyb_init(): Couldn't tcgetattr(kbd_fd,...) errno=%d\n", errno);
+      leavedos(66);
+    }
   }
 
   buf = save_termios;
@@ -1346,7 +1373,7 @@ int slang_keyb_init(void) {
   cfgetispeed(&buf);
   cfgetospeed(&buf);
 #endif
-  if (tcsetattr(kbd_fd, TCSANOW, &buf) < 0) {
+  if (tcsetattr(kbd_fd, TCSANOW, &buf) < 0 && errno != EINVAL) {
     error("slang_keyb_init(): Couldn't tcsetattr(kbd_fd,TCSANOW,...) !\n");
   }
 
@@ -1385,8 +1412,8 @@ int slang_keyb_init(void) {
   return TRUE;
 }
 
-void slang_keyb_close(void)  {
-   if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1) {
+static void n_slang_keyb_close(void)  {
+   if (tcsetattr(kbd_fd, TCSAFLUSH, &save_termios) == -1  && errno != EINVAL) {
       error("slang_keyb_close(): failed to restore keyboard termios settings!\n");
    }
    if (save_kbd_flags != -1) {
@@ -1394,11 +1421,26 @@ void slang_keyb_close(void)  {
    }
 }
 
+int slang_keyb_init(void);
+
 struct keyboard_client Keyboard_slang =  {
    "slang",                    /* name */
    slang_keyb_init,            /* init */
    NULL,                       /* reset */
-   slang_keyb_close,           /* close */
-   do_slang_getkeys            /* run */
+   n_slang_keyb_close,           /* close */
+   n_do_slang_getkeys            /* run */
 };
    
+int slang_keyb_init(void)
+{
+  extern o_slang_keyb_init(void);
+  extern void o_do_slang_getkeys(void);
+  extern void o_slang_keyb_close(void);
+  if (config.features[0]) {
+    Keyboard_slang.close = &o_slang_keyb_close;
+    Keyboard_slang.run = &o_do_slang_getkeys;
+    return o_slang_keyb_init();
+  }
+  return n_slang_keyb_init();
+}
+
