@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -1393,6 +1393,101 @@ CONST unsigned char alt_map_pl[] =
   0, 0, 0, 0, 0, 0, 0, 0,
   0};
 
+CONST unsigned char key_map_hr_cp852[] =
+{
+  0, 27, '1', '2', '3', '4', '5', '6',
+  '7', '8', '9', '0', '\'', '+', 127, 9,
+  'q', 'w', 'e', 'r', 't', 'z', 'u', 'i',
+  'o', 'p', 231, 208, 13, 0, 'a', 's',     /*231=s, 208=d*/
+  'd', 'f', 'g', 'h', 'j', 'k', 'l', 159,  /*159=c tvrdo*/
+  134, 0, 0, 167, 'y', 'x', 'c', 'v',      /*134=c meko,167=z*/
+  'b', 'n', 'm', ',', '.', '-', 0, '*',
+  0, 32, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, '-', 0, 0, 0, '+', 0,
+  0, 0, 0, 0, 0, 0, '<', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
+
+CONST unsigned char shift_map_hr_cp852[] =
+{
+  0, 27, '!', '\"', '#', '$', '%', '&',
+  '/', '(', ')', '=', '?', '*', 127, 9,
+  'Q', 'W', 'E', 'R', 'T', 'Z', 'U', 'I',
+  'O', 'P', 230, 209, 13, 0, 'A', 'S',     /*230=S, 209=D*/
+  'D', 'F', 'G', 'H', 'J', 'K', 'L', 172,  /*172=C tvrdo*/
+  143, 0, 0, 166, 'Y', 'X', 'C', 'V',      /*143=C meko,166=Z*/
+  'B', 'N', 'M', ';', ':', '_', 0, '*',
+  0, 32, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, '-', 0, 0, 0, '+', 0,
+  0, 0, 0, 0, 0, 0, '>', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
+
+CONST unsigned char alt_map_hr_cp852[] =
+{
+  0, 0, 126, 183, '^', 244, 248, 242,
+  '`', 250, 239, 241, 249, 247, 0, 0,
+  '\\', '|', 0, 0, 0, 0, 0, 0,
+  0, 0, 246, 158, 13, 0, 0, 0,
+  0, '[', ']', 0, 0, 146, 145, '\\',
+  225, 0, 0, 207, 0, 0, 0, '@',
+  '{', '}', 245, 0, '|', '/', 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, '|', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
+
+CONST unsigned char key_map_hr_latin2[] =
+{
+  0, 27, '1', '2', '3', '4', '5', '6',
+  '7', '8', '9', '0', '\'', '+', 127, 9,
+  'q', 'w', 'e', 'r', 't', 'z', 'u', 'i',
+  'o', 'p', 185, 240, 13, 0, 'a', 's',     /*185=s, 240=d*/
+  'd', 'f', 'g', 'h', 'j', 'k', 'l', 232,  /*232=c tvrdo*/
+  230, 0, 0, 190, 'y', 'x', 'c', 'v',      /*230=c meko,190=z*/
+  'b', 'n', 'm', ',', '.', '-', 0, '*',
+  0, 32, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, '-', 0, 0, 0, '+', 0,
+  0, 0, 0, 0, 0, 0, '<', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
+
+CONST unsigned char shift_map_hr_latin2[] =
+{
+  0, 27, '!', '\"', '#', '$', '%', '&',
+  '/', '(', ')', '=', '?', '*', 127, 9,
+  'Q', 'W', 'E', 'R', 'T', 'Z', 'U', 'I',
+  'O', 'P', 169, 208, 13, 0, 'A', 'S',     /*169=S, 208=D*/
+  'D', 'F', 'G', 'H', 'J', 'K', 'L', 200,  /*200=C tvrdo*/
+  198, 0, 0, 174, 'Y', 'X', 'C', 'V',      /*198=C meko,174=Z*/
+  'B', 'N', 'M', ';', ':', '_', 0, '*',
+  0, 32, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, '-', 0, 0, 0, '+', 0,
+  0, 0, 0, 0, 0, 0, '>', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
+
+CONST unsigned char alt_map_hr_latin2[] =
+{
+  0, 0, 126, 183, '^', 162, 176, 178,
+  '`', 255, 180, 189, 168, 184, 0, 0,
+  '\\', '|', 0, 0, 0, 0, 0, 0,
+  0, 0, 247, 215, 13, 0, 0, 0,
+  0, '[', ']', 0, 0, 179, 163, '\\',
+  223, 0, 0, 164, 0, 0, 0, '@',
+  '{', '}', 167, 0, '|', '/', 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, '|', 0,
+  0, 0, 0, 0, 0, 0, 0, 0,
+  0};
 
 /* this prefereable is overloaded via '-I keytable keyb-user'
  * and is preset with an US keyboard layout
@@ -1685,6 +1780,14 @@ struct keytable_entry keytable_list[] = {
     key_map_pl, shift_map_pl, alt_map_pl,
     num_table_comma, dead_key_table,
     dos852_dead_map},
+  {"hr-cp852", KEYB_HR_CP852, KT_USES_ALTMAP, sizeof(key_map_hr_cp852), sizeof(num_table_comma),
+    key_map_hr_cp852, shift_map_hr_cp852, alt_map_hr_cp852,
+    num_table_comma, dead_key_table,
+    dos850_dead_map},
+  {"hr-latin2", KEYB_HR_LATIN2, KT_USES_ALTMAP, sizeof(key_map_hr_latin2), sizeof(num_table_comma),
+    key_map_hr_latin2, shift_map_hr_latin2, alt_map_hr_latin2,
+    num_table_comma, dead_key_table,
+    dos850_dead_map},
   {"keyb-user", KEYB_USER, 0, sizeof(key_map_user), sizeof(num_table_comma),
     key_map_user, shift_map_user, alt_map_user,
     num_table_dot, dead_key_table,
