@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /* dos emulator, Matthias Lautner
  * Extensions by Robert Sanders, 1992-93
  *
@@ -25,6 +31,14 @@
 #include "emu.h"
 #include "disks.h"
 #include "priv.h"
+#include "dosemu_select.h"
+
+#ifdef NEED_LLSEEK_PROTOTYPE
+  /* well, if we don't have llseek prototype,
+   * we most likely won't have __loff_t too, hence using long long
+   */
+  extern long long llseek (int fd, long long offset, int origin);
+#endif
 
 static int disks_initiated = 0;
 
@@ -655,14 +669,9 @@ disk_open(struct disk *dp)
    * during the ioctl. (well, not what we really like)
    * ( 19 May 1996, Hans Lermen ) */
   int res;
- #ifdef USE_THREADS
-  static int background_ioctl(int fd, int request, void *param);
-  res = background_ioctl(dp->fdesc, FDGETPRM, &fl);
- #else
   sigalarm_onoff(0);
   res = ioctl(dp->fdesc, FDGETPRM, &fl);
   sigalarm_onoff(1);
- #endif
   if (res == -1) {
 #else
   if (ioctl(dp->fdesc, FDGETPRM, &fl) == -1) {
@@ -1051,7 +1060,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly) {
-      error("write protect!\n");
+      W_printf("write protect!\n");
       show_regs(__FILE__, __LINE__);
       if (dp->removeable)
 	HI(ax) = DERR_WP;
@@ -1062,7 +1071,7 @@ int13(u_char i)
     }
 
     if (dp->rdonly)
-      error("CONTINUED!!!!!\n");
+      W_printf("CONTINUED!!!!!\n");
     res = write_sectors(dp, buffer, head, sect, track, number);
 
     if (res < 0) {
@@ -1503,7 +1512,7 @@ int13(u_char i)
     NOCARRY;
     break;
   default:
-    error("disk error, unknown command: int13, ax=0x%x\n",
+    d_printf("disk error, unknown command: int13, ax=0x%x\n",
 	  LWORD(eax));
     show_regs(__FILE__, __LINE__);
     CARRY;
@@ -1527,48 +1536,3 @@ floppy_tick(void)
   }
 }
 
-#ifdef USE_THREADS
-
-#include "lt-threads.h"
-
-static mbox_handle ioctlmbox_out;
-static mbox_handle ioctlmbox_in;
-
-struct {
-  struct msg msg;
-  int fd;
-  int request;
-  void *param;
-  int result;
-} ioctlmsg;
-
-void background_ioctl_thread(int start)
-{
-  if (!start) {
-    /* we come here _before_ any thread is started
-     * but threading is active and we run at scope of thread0.
-     * We now create the resource we need.
-     */
-     ioctlmbox_in = create_local_mailbox("diskioctl_in", 2);
-     ioctlmbox_out = create_local_mailbox("diskioctl_out", 2);
-     return;
-  }
-  /* here we are a separate thread */
-  while (1) {
-    receivemessage(ioctlmbox_in);
-    ioctlmsg.result = ioctl(ioctlmsg.fd, ioctlmsg.request, ioctlmsg.param);
-    sendmessage(ioctlmbox_out,0);
-  }
-}
-
-static int background_ioctl(int fd, int request, void *param)
-{
-  ioctlmsg.fd = fd;
-  ioctlmsg.request = request;
-  ioctlmsg.param = param;
-  sendmessage(ioctlmbox_in,0);
-  receivemessage(ioctlmbox_out);
-  return ioctlmsg.result;
-}
-
-#endif
