@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -22,13 +22,13 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/vm86.h>
-#if defined (__GLIBC__) && __GLIBC__ >=2
+#if GLIBC_VERSION_CODE >= 2000
 #  include <netipx/ipx.h>
 #else
 #  include <linux/sockios.h>
 #  include <asm/types.h>
 #  include <linux/ipx.h>
-#endif /* __GLIBC >= 2 */
+#endif /* GLIBC_VERSION_CODE */
 #include <netinet/in.h>
 #include <errno.h>
 
@@ -38,6 +38,7 @@
 #include "cpu.h"
 #include "inifile.h"
 #include "doshelpers.h"
+#include "dosemu_select.h"
 
 #define MAX_PACKET_DATA		1500
 
