@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -42,7 +42,7 @@
 #include "netbsd_vm86.h"
 #endif
 #ifdef __linux__
-#if __GLIBC__ > 1
+#if GLIBC_VERSION_CODE >= 2000
 #include <sys/vt.h>
 #else
 #include <linux/vt.h>
