@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -89,7 +89,7 @@ __asm__("___START___: jmp _emulate\n");
 #include "netbsd_vm86.h"
 #endif
 #ifdef __linux__
-#if __GLIBC__ > 1
+#if GLIBC_VERSION_CODE >= 2000
 #include <sys/vt.h>
 #include <sys/kd.h>
 #else
@@ -377,10 +377,10 @@ changesegs()
  * 
  */
 #ifdef __ELF__
-void 
+int
 main(int argc, char **argv)
 #else
-void 
+int
 emulate(int argc, char **argv)
 #endif
 {
@@ -468,6 +468,7 @@ emulate(int argc, char **argv)
     sync();
     fprintf(stderr, "Not a good day to die!!!!!\n");
     leavedos(99);
+    return 0;  /* just to make gcc happy */
 }
 
 #if 0    /* disable C-A-D until someone will fix it (if really needed) */
