@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1998 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -42,12 +42,11 @@
  * int10()
  *
  * added:
- *  vga256_cmap_init()
+ *  graphics_cmap_init()
  *
  * Now dosemu can work in graphics mode with X, but at the moment only in mode
  * 0x13 and very slow.
  *
- *
  * 1995-08-30: Merged with code for pasting, removed some restrictions 
  * from Pasi's code: cut and paste and x_mouse, 
  * (bon@elektron.ikp.physik.th-darmstadt.de)
@@ -123,26 +122,43 @@
  * 1998/05/28: some re-arangements of the mouse code.
  * -- EB
  *
+ * 1998/09/16: Continued work on color allocation.
+ * Added graphics support for 16 color X servers (e.g. XF86_VGA16).
+ * Text mode now works on all X servers incl. 16 color & mono servers.
+ * -- sw
+ *
+ * 1998/10/24: Removed quite a lot of DAC/color init code.
+ * -- sw
  * DANG_END_CHANGELOG
  */
 
 
-#define X_USE_BACKING_STORE		/* use backing store in text modes */
+/* use backing store in text modes */
+#define X_USE_BACKING_STORE
 
-#define CONFIG_X_SELECTION 1
-#define CONFIG_X_MOUSE 1
-#define CONFIG_X_SPEAKER 1  		/* do you want speaker support in X? */
+/* Do you want speaker support in X? */
+#define CONFIG_X_SPEAKER 1
 
-/* undef no longer works -- 1998/03/15 sw */
-#define TEXT_DAC_UPDATES		/* updates palettes even in text modes */
+/* show X mouse in graphics modes */
+#undef DEBUG_MOUSE_POS
 
-#undef DEBUG_MOUSE_POS			/* show X mouse in graphics modes */
-
-#undef ENABLE_KEYBOARD_GRAB		/* when grabbing all mouse events, grab keyboard too */
+/* when grabbing all mouse events, grab keyboard too */
+#undef ENABLE_KEYBOARD_GRAB
 
 /* define if you wish to have a visual indication of the area being updated */
 #undef DEBUG_SHOW_UPDATE_AREA
 
+/* "fine tuning" options for X_update_screen */
+#define MAX_UNCHANGED	3
+
+/* not yet -- sw */
+#undef HAVE_DGA
+
+#define CONFIG_X_SELECTION 1
+
+#define CONFIG_X_MOUSE 1
+
+
 #include <config.h>
 #include <stdio.h>
 #include <termios.h>
@@ -172,7 +188,6 @@
 #include <X11/extensions/XShm.h>
 #endif
 
-#undef HAVE_DGA		/* not yet -- sw */
 #ifdef HAVE_DGA
 #include <X11/extensions/xf86dga.h>
 #endif
@@ -260,9 +275,6 @@ extern char **dosemu_argv;
 /* From Xkeyb.c */
 extern void X_process_key(XKeyEvent *);
 
-/* "Fine tuning" options for X_update_screen */
-#define MAX_UNCHANGED	3
-
 /* Kludge for incorrect ASCII 0 char in vga font. */
 #define XCHAR(w) ((u_char)CHAR(w)?(u_char)CHAR(w):(u_char)' ')
 
@@ -300,11 +312,11 @@ static unsigned _dga_height = 0;
 static Window dga_window;
 #endif
 
-Display *display;
+Display *display;		/* used by base/keyboard/keyb_X_keycode.c */
 static int screen;
+static Visual *visual;
 static Window rootwindow, mainwindow;
 static XImage *ximage;		/*Used as a buffer for the X-server*/
-static XColor xcol;
 
 static Cursor X_standard_cursor;
 
@@ -337,13 +349,12 @@ static Boolean doing_selection = FALSE, visible_selection = FALSE;
 static Boolean have_focus = FALSE;
 static Boolean is_mapped = FALSE;
 
-static Colormap text_cmap = 0;
+static int cmap_colors = 0;			/* entries in colormaps: {text,graphics}_cmap */
+static Colormap text_cmap = 0, graphics_cmap = 0;
+static Boolean have_shmap = FALSE;
 static unsigned long text_colors[16];
 static int text_col_stats[16] = {0};
 
-static Colormap vga256_cmap = 0;
-static Boolean have_shmap = FALSE;
-
 static RemapObject remap_obj;
 static ColorSpaceDesc X_csd;
 static int have_true_color;
@@ -376,16 +387,14 @@ static void text_cmap_init(void);
 static int try_cube(unsigned long *, c_cube *);
 static ColorSpaceDesc MakeSharedColormap(void);
 static int MakePrivateColormap(void);
-static void vga256_cmap_init(void);
+static void graphics_cmap_init(void);
 static void load_text_font(void);
 static void load_cursor_shapes(void);
-static void InitDACTable(void);
 static void refresh_private_palette(void);
 static void refresh_truecolor(void);
 static void refresh_palette(void);
-#ifdef TEXT_DAC_UPDATES
+static void get_approx_color(XColor *, Colormap, int);
 static void refresh_text_palette(void);
-#endif
 static void put_ximage(int, int, int, int, unsigned, unsigned);
 static int NewXErrorHandler(Display *, XErrorEvent *);
 static void create_ximage(void);
@@ -410,14 +419,14 @@ static void draw_cursor(int, int);
 static inline void restore_cell(int, int);
 static void redraw_cursor(void);
 static void X_update_cursor(void);
-extern void X_blink_cursor(void);
+void X_blink_cursor(void);
 void X_redraw_screen(void);
 #if USE_SCROLL_QUEUE
 static void X_scroll(int, int, int, int, int, byte);
 static void do_scroll(void);
 #endif
 int X_update_screen(void);
-extern void X_change_mouse_cursor(void);
+void X_change_mouse_cursor(void);
 #if CONFIG_X_MOUSE
 static void set_mouse_position(int, int);
 static void set_mouse_buttons(int);
@@ -436,7 +445,7 @@ static void save_selection_data(void);
 static void end_selection(void);
 static void send_selection(Time, Window, Atom, Atom);
 #endif
-extern void X_handle_events(void);
+void X_handle_events(void);
 
 
 
@@ -444,63 +453,6 @@ extern void X_handle_events(void);
 /*                         INITIALIZATION                                 */
 /**************************************************************************/
 
-#if 0
-/*
- * Allocate normal VGA colors, and store color indices to text_colors[].
- */
-static void text_cmap_init()
-{
-  int i;
-  XColor xcol2;
-
-  /* The following are almost IBM standard color codes, with some slight
-   * gamma correction for the dim colors to compensate for bright X
-   * screens.
-  */
-  c_cube crgb[16] = {
-    {0x00,0x00,0x00}, {0x18,0x18,0xB2}, {0x18,0xB2,0x18}, {0x18,0xB2,0xB2},
-    {0xB2,0x18,0x18}, {0xB2,0x18,0xB2}, {0xB2,0x68,0x18}, {0xB2,0xB2,0xB2},
-    {0x68,0x68,0x68}, {0x54,0x54,0xFF}, {0x54,0xFF,0x54}, {0x54,0xFF,0xFF},
-    {0xFF,0x54,0x54}, {0xFF,0x54,0xFF}, {0xFF,0xFF,0x54}, {0xFF,0xFF,0xFF}
-  };
-
-  text_cmap = DefaultColormap(display, screen);
-
-  X_printf("X: text_cmap_init: getting VGA colors\n");
-
-  /* Since we're here, we may as well set this up since it never changes... */
-  xcol.flags = DoRed | DoGreen | DoBlue;
-
-  for(i = 0; i < 16; i++) {
-    xcol2.red   = crgb[i].r * 0x101;
-    xcol2.green = crgb[i].g * 0x101;
-    xcol2.blue  = crgb[i].b * 0x101;
-    text_col_stats[i] = XAllocColor(display, text_cmap, &xcol2);
-    text_colors[i] = xcol2.pixel;
-
-    if(! text_col_stats[i]) {
-      X_printf("X: text_cmap_init: couldn't allocate color %d\n", i);
-    }
-    X_printf("X: text_cmap_init: color[%d] = %d\n", i, (int) text_colors[i]);
-  }
-}
-#endif
-
-c_cube c_cubes[] = {
-  { 6, 8, 5 },
-  { 6, 7, 5 },
-  { 6, 6, 5 },
-  { 5, 7, 5 },
-  { 5, 6, 5 },
-  { 4, 8, 4 },
-  { 5, 6, 4 },
-  { 5, 5, 4 },
-  { 4, 5, 4 },
-  { 4, 5, 3 },
-  { 4, 4, 3 },
-  { 3, 4, 3 }
-};
-
 int try_cube(unsigned long *p, c_cube *c)
 {
   unsigned r, g, b;
@@ -513,11 +465,11 @@ int try_cube(unsigned long *p, c_cube *c)
     xc.red = (0xffff * r) / c->r;
     xc.green = (0xffff * g) / c->g;
     xc.blue = (0xffff * b) / c->b;
-    if(XAllocColor(display, vga256_cmap, &xc)) {
+    if(XAllocColor(display, graphics_cmap, &xc)) {
       p[i++] = xc.pixel;
     }
     else {
-      while(--i >= 0) XFreeColors(display, vga256_cmap, p + i, 1, 0);
+      while(--i >= 0) XFreeColors(display, graphics_cmap, p + i, 1, 0);
       return 0;
     }
   }
@@ -530,6 +482,11 @@ ColorSpaceDesc MakeSharedColormap()
   XColor xc;
   int i, j;
   static unsigned long pix[256];
+  c_cube c_cubes[] = {
+    { 6, 8, 5 }, { 6, 7, 5 }, { 6, 6, 5 }, { 5, 7, 5 },
+    { 5, 6, 5 }, { 4, 8, 4 }, { 5, 6, 4 }, { 5, 5, 4 },
+    { 4, 5, 4 }, { 4, 5, 3 }, { 4, 4, 3 }, { 3, 4, 3 }
+  };
 
   xc.flags = DoRed | DoGreen | DoBlue;
 
@@ -569,64 +526,68 @@ ColorSpaceDesc MakeSharedColormap()
   return csd;
 }
 
+
+/*
+ * Create a private colormap. The colormap is *not* initialized.
+ */
 int MakePrivateColormap()
 {
   int i;
-  DAC_entry color;
-  unsigned long pixels[256];
+  unsigned long pixels[256];            /* ok, cmap_colors always <= 256 */
 
-  vga256_cmap = XCreateColormap(display, rootwindow, DefaultVisual(display, screen), AllocNone);
+  graphics_cmap = XCreateColormap(display, rootwindow, visual, AllocNone);
 
-  i = XAllocColorCells(display, vga256_cmap, True, NULL, 0, pixels, 256);
+  i = XAllocColorCells(display, graphics_cmap, True, NULL, 0, pixels, cmap_colors);
   if(!i) {
-    X_printf("X: MakePrivateColormap: failed to allocate private color map\n");
+    X_printf("X: failed to allocate private color map (no PseudoColor visual)\n");
     return 0;
   }
 
-  for(i = 0; i < 256; i++) {
-    DAC_get_entry(&color, (unsigned char) i);
-    xcol.pixel = i;
-    xcol.red   = (color.r * 65535) / 63;
-    xcol.green = (color.g * 65535) / 63;
-    xcol.blue  = (color.b * 65535) / 63;
-    XStoreColor(display, vga256_cmap, &xcol);
-  }
-
   return 1;
 }
 
 
-/* DANG_BEGIN_FUNCTION vga256_cmap_init
+/*
+ * DANG_BEGIN_FUNCTION graphics_cmap_init
  *
- * Allocates a colormap for 256 color modes and initializes it.
+ * description:
+ * Allocate a colormap for graphics modes and initialize it.
+ * Do mostly nothing on true color displays.
+ * Otherwise, do:
+ *   - if colormaps have less than 256 entries (notably 16 or 2 colors),
+ *     don't use a private colormap
+ *   - if a shared map is requested and there are less than 36 colors (3/4/3)
+ *     available, use a private colormap
+ *
+ * Note: Text modes always use the screen's default colormap.
  *
  * DANG_END_FUNCTION
+ *
+ * Note: The above is not fully true. If on 16 or 2 color displays the
+ * allocation of a shared map fails for some reason, the code still tries
+ * to get a private map (and will likely fail). -- sw
  */
-static void vga256_cmap_init(void)
+void graphics_cmap_init()
 {
   /* Note: should really deallocate and reallocate our colours every time
    we switch graphic modes... --adm */
 
-  DAC_init();
-
   if(have_true_color) return;
 
-  if(vga256_cmap == 0) {	/* only the first time! */
+  if(graphics_cmap == 0) {	/* only the first time! */
     have_shmap = FALSE;
 
-    if(config.X_sharecmap) {
-      vga256_cmap = DefaultColormap(display, screen);
+    if(config.X_sharecmap || cmap_colors < 256) {		/* notably on 16 color and mono screens */
+      graphics_cmap = DefaultColormap(display, screen);
       X_csd = MakeSharedColormap();
 
       if(X_csd.bits == 1) {
-        X_printf("X: vga256_cmap_init: couldn't get enough free colors; trying private colormap\n");
-        /* this is certainly not an error, hence no  error() call -- sw */
-        fprintf(stderr, "X: Couldn't get enough free colors - using private colormap.\n");
+        X_printf("X: graphics_cmap_init: couldn't get enough free colors; trying private colormap\n");
         have_shmap = FALSE;
       }
       else {
         if(X_csd.bits < 4 * 5 * 4) {
-          X_printf("X: vga256_cmap_init: couldn't get many free colors (%d). May look bad.\n", X_csd.bits);
+          X_printf("X: graphics_cmap_init: couldn't get many free colors (%d). May look bad.\n", X_csd.bits);
         }
       }
 
@@ -635,18 +596,19 @@ static void vga256_cmap_init(void)
 
     if(!have_shmap) {
       if(MakePrivateColormap()) {
-        X_printf("X: vga256_cmap_init: using private colormap.\n");
+        X_printf("X: graphics_cmap_init: using private colormap.\n");
       }
       else {
-        error("X: Warning: we don't have any kind of colormap!\n");
+        graphics_cmap = 0;
+        error("X: graphics_cmap_init: Couldn't get a colormap for graphics modes!\n");
       }
     }
     else {
-      X_printf("X: vga256_cmap_init: using shared colormap.\n");
+      X_printf("X: graphics_cmap_init: using shared colormap.\n");
     }
   }
   else {
-    X_printf("X: vga256_cmap_init: unexpectedly called\n");
+    X_printf("X: graphics_cmap_init: unexpectedly called\n");
   }
 }
 
@@ -746,20 +708,29 @@ static void load_cursor_shapes(void)
 
 /* true color stuff >>>>>>>>>>>>>>>>>>>> */
 
+#if 0
 /*
- * keep color map up to date
+ * The following code is currently *NEVER* called. Maybe this
+ * will change sometimes, so I keep it for now.
+ * Note that this made the function pixel2RGB() redundant as well.
+ * -- sw
  */
 
-void InitDACTable()
+/*
+ * Keep colormap up to date. Typically called when entering
+ * a graphics mode to initialize the whole colormap.
+ */
+void update_all_colors()
 {
-  DAC_entry dac;
+  DAC_entry c;
   int i;
 
-  for(i = 0; i < 256; i++) {
-    DAC_get_entry(&dac, (unsigned char) i);
-    remap_obj.palette_update(&remap_obj, i, dac_bits, dac.r, dac.g, dac.b);
+  for(i = 0; i < cmap_colors; i++) {
+    pixel2RGB(i, &c);
+    remap_obj.palette_update(&remap_obj, i, dac_bits, c.r, c.g, c.b);
   }
 }
+#endif
 
 
 static void refresh_private_palette()
@@ -768,10 +739,16 @@ static void refresh_private_palette()
   RGBColor c;
   XColor xcolor[256];
   int i, n;
-  unsigned bits;
+  unsigned bits, shift;
 
-  for (n = 0; ((i = DAC_get_dirty_entry(&color)) != -1) && n < 256; n++) {
-    unsigned shift;
+
+  /*
+   * Colormap size is limited to cmap_colors, not 256. This leads to
+   * incorrect displays at screens with less than 8 bit color depth
+   * 
+   */
+
+  for(n = 0; ((i = pixel2RGB_dirty(&color)) != -1) && n < cmap_colors; n++) {
     c.r = color.r; c.g = color.g; c.b = color.b;
     bits = dac_bits;
     gamma_correct(&remap_obj, &c, &bits);
@@ -789,7 +766,7 @@ static void refresh_private_palette()
 	     xcolor[n].blue);
   }
   /* Hopefully this is faster */
-  XStoreColors(display, vga256_cmap, xcolor, n);
+  if(graphics_cmap) XStoreColors(display, graphics_cmap, xcolor, n);
 }
 
 static void refresh_truecolor()
@@ -798,7 +775,7 @@ static void refresh_truecolor()
   Boolean colchanged = False;
   int i;
 
-  while((i = DAC_get_dirty_entry(&color)) != -1) {
+  while((i = pixel2RGB_dirty(&color)) != -1) {
     colchanged |= remap_obj.palette_update(&remap_obj, i, dac_bits, color.r, color.g, color.b);
   }
   if(colchanged) dirty_all_video_pages();
@@ -807,53 +784,116 @@ static void refresh_truecolor()
 
 static void refresh_palette()
 {
-#ifdef TEXT_DAC_UPDATES
-  if (vga.mode_class == TEXT)
-	  refresh_text_palette();
-  else
-#endif
-  if(have_true_color || have_shmap)
-    refresh_truecolor();
-  else
-    refresh_private_palette();
+  if(vga.mode_class == TEXT) {
+    refresh_text_palette();
+  }
+  else {
+    if(have_true_color || have_shmap)
+      refresh_truecolor();
+    else
+      refresh_private_palette();
+  }
+}
+
+
+/*
+ * Scans the colormap cmap for the color closest to xc and
+ * returns it in xc. The color values used by cmap are queried
+ * from the X server only if read_cmap is set.
+ */
+
+static void get_approx_color(XColor *xc, Colormap cmap, int read_cmap)
+{
+  int i, d0, ind;
+  unsigned d1, d2;
+  static XColor xcols[256];
+
+  if(read_cmap) {
+    for(i = 0; i < cmap_colors; i++) xcols[i].pixel = i;
+    XQueryColors(display, cmap, xcols, cmap_colors);
+  }
+
+  ind = -1; d2 = -1;
+  for(i = 0; i < cmap_colors; i++) {
+    /* Note: X color values are unsigned short, so using int for the sum (d1) should be ok. */
+    d0 = (int) xc->red - xcols[i].red; d1 = abs(d0);
+    d0 = (int) xc->green - xcols[i].green; d1 += abs(d0);
+    d0 = (int) xc->blue - xcols[i].blue; d1 += abs(d0);
+    if(d1 < d2) ind = i, d2 = d1;
+  }
+
+  if(ind >= 0) *xc = xcols[ind];
 }
 
-#ifdef TEXT_DAC_UPDATES
 static void refresh_text_palette()
 {
   DAC_entry color;
   XColor xc;
   int i, n, shift = 16 - dac_bits;
+  int read_cmap = 1;
 
-  for (n = 0; ((i = DAC_get_dirty_entry(&color)) != -1) && n < 16 && i < 16; n++) {
+  for (n = 0; ((i = pixel2RGB_dirty(&color)) != -1) && n < 16 && i < 16; n++) {
     xc.flags = DoRed | DoGreen | DoBlue;
     xc.pixel = text_colors[i];
     xc.red   = color.r << shift;
     xc.green = color.g << shift;
     xc.blue  = color.b << shift;
 
-    if(text_col_stats[i]) XFreeColors(display, text_cmap, &(xc.pixel), 1, 0);
+    if(text_col_stats[i]) XFreeColors(display, text_cmap, &xc.pixel, 1, 0);
 
     if(!(text_col_stats[i] = XAllocColor(display, text_cmap, &xc))) {
-      printf("X: refresh_text_palette: failed to allocate new text color: %d\n", i);
+      get_approx_color(&xc, text_cmap, read_cmap);
+      read_cmap = 0;
+      X_printf("X: refresh_text_palette: %d (%d -> app. %d)\n", i, (int) text_colors[i], (int) xc.pixel);
+      
+    }
+    else {
+      X_printf("X: refresh_text_palette: %d (%d -> %d)\n", i, (int) text_colors[i], (int) xc.pixel);
     }
-    X_printf("X: refresh_text_palette: %d --> %d\n", (int) text_colors[i], (int) xc.pixel);
     text_colors[i] = xc.pixel;
   }
 }
-#endif
 
 static void X_get_screen_info()
 {
   XImage *xi;
-  Visual *xdv;
+  char *s;
 
-  xdv = DefaultVisual(display, DefaultScreen(display));
-  have_true_color = xdv->class == TrueColor || xdv->class == DirectColor ? 1 : 0;
+  screen = DefaultScreen(display);
+  rootwindow = RootWindow(display, screen);
+  visual = DefaultVisual(display, DefaultScreen(display));
+
+  have_true_color = visual->class == TrueColor || visual->class == DirectColor ? 1 : 0;
+
+  switch(visual->class) {
+    case StaticGray:  s = "StaticGray"; break;
+    case GrayScale:   s = "GrayScale"; break;
+    case StaticColor: s = "StaticColor"; break;
+    case PseudoColor: s = "PseudoColor"; break;
+    case TrueColor:   s = "TrueColor"; break;
+    case DirectColor: s = "DirectColor"; break;
+    default:          s = "Unknown";
+  }
+  X_printf("X: visual class is %s\n", s);
+
+  if(have_true_color) {
+    X_printf("X: using true color visual\n");
+  }
+  else {
+    int i = DefaultDepth(display, DefaultScreen(display));
+
+    if(i > 8) i = 8;				/* we will never try to get more */
+    if((cmap_colors = 1 << i) == 1) {
+      X_printf("X: Oops, your screen has only *one* color?\n");
+    }
+    else {
+      X_printf("X: colormaps have %d colors\n", cmap_colors);
+    }
+  }
 
   xi = XCreateImage(
     display,
-    DefaultVisual(display, DefaultScreen(display)),
+    visual,
     DefaultDepth(display, DefaultScreen(display)),
     ZPixmap, 0, NULL, 6 * 8, 2, 32, 0
   );
@@ -865,21 +905,21 @@ static void X_get_screen_info()
     }
     else {
       ximage_bits_per_pixel = xi->bits_per_pixel;
-      X_printf("X: could not determine pixel size, assuming %d bits/pixel\n", ximage_bits_per_pixel);
+      X_printf("X: could not determine pixel size, guessing %d bits/pixel\n", ximage_bits_per_pixel);
     }
 
     XDestroyImage(xi);
   }
   else {
     ximage_bits_per_pixel = DefaultDepth(display, DefaultScreen(display));
-    X_printf("X: could not determine pixel size, assuming %d bits/pixel\n", ximage_bits_per_pixel);
+    X_printf("X: could not determine pixel size, guessing %d bits/pixel\n", ximage_bits_per_pixel);
   }
 
   X_csd.bits = ximage_bits_per_pixel;
-  X_csd.bytes = (ximage_bits_per_pixel + 7) >>3;
-  X_csd.r_mask = xdv->red_mask;
-  X_csd.g_mask = xdv->green_mask;
-  X_csd.b_mask = xdv->blue_mask;
+  X_csd.bytes = (ximage_bits_per_pixel + 7) >> 3;
+  X_csd.r_mask = visual->red_mask;
+  X_csd.g_mask = visual->green_mask;
+  X_csd.b_mask = visual->blue_mask;
   color_space_complete(&X_csd);
   if(X_csd.bits == 16 && (X_csd.r_bits + X_csd.g_bits + X_csd.b_bits) == 15) {
     X_csd.bits = ximage_bits_per_pixel = 15;
@@ -929,7 +969,7 @@ void create_ximage()
   if(shm_ok) {
     ximage = XShmCreateImage(
       display,
-      DefaultVisual(display,DefaultScreen(display)),
+      visual,
       DefaultDepth(display,DefaultScreen(display)),
       ZPixmap, NULL,
       &shminfo,
@@ -971,7 +1011,7 @@ void create_ximage()
   {
     ximage = XCreateImage(
       display,
-      DefaultVisual(display,DefaultScreen(display)),
+      visual,
       DefaultDepth(display,DefaultScreen(display)),
       ZPixmap, 0, NULL,
       w_x_res, w_y_res,
@@ -1139,7 +1179,6 @@ int X_init(void)
   X_printf("X: X_init\n");
 
   co = 80; li = 25;
-
   set_video_bios_size();		/* make it stick */
 
   /* Open X connection. */
@@ -1150,9 +1189,8 @@ int X_init(void)
     leavedos(99);
   }
 
-  screen = DefaultScreen(display);
-  rootwindow = RootWindow(display,screen);
-  text_cmap = DefaultColormap(display, screen);
+  /* collect some data about our X server; do it before anything else */
+  X_get_screen_info();
 
   OldXErrorHandler = XSetErrorHandler(NewXErrorHandler);
 
@@ -1164,19 +1202,12 @@ int X_init(void)
   X_dga_init();
 #endif
 
-  X_get_screen_info();		/* do it _before_ vga256_cmap_init() */
-
-#if 0
-  /* what for ? -- sw */
-  text_cmap_init();		/* text mode colors */
-#endif
-
-  vga256_cmap_init();		/* 8 bit mode colors */
+  text_cmap = DefaultColormap(display, screen);
+  graphics_cmap_init();		/* 8 bit mode colors */
 
   X_remap_init();		/* init graphics mode support */
-
   if(!remap_src_modes) {
-    error("X: Warning: no graphics modes supported on this type of screen!\n");
+    error("X: No graphics modes supported on this type of screen!\n");
   }
 
   load_text_font();
@@ -1191,6 +1222,10 @@ int X_init(void)
     text_colors[0]		/* Background */
   );
 
+  X_printf("X: X_init: screen = %d, root = 0x%x, mainwindow = 0x%x\n",
+    screen, (unsigned) rootwindow, (unsigned) mainwindow
+  );
+
   load_cursor_shapes();
 
   gcv.font = vga_font;
@@ -1231,18 +1266,8 @@ int X_init(void)
     );
   }
 
-
-  {
-    /* just for testing purposes; this still lacks some proper configuration strategy
-     * -- 1998/03/08 sw
-     */
-    if(getenv("DOSEMU_HIDE_WINDOW") == NULL) XMapWindow(display, mainwindow);
-  }
-
-
-  X_printf("X: X_init done, screen = %d, root = 0x%lx, mainwindow = 0x%lx\n",
-    screen, (unsigned long) rootwindow, (unsigned long) mainwindow
-  );
+  /* don't map window if set */
+  if(getenv("DOSEMU_HIDE_WINDOW") == NULL) XMapWindow(display, mainwindow);
 
   /* initialize VGA emulator */
   X_screen.src_modes = remap_src_modes;
@@ -1271,14 +1296,14 @@ int X_init(void)
     X_printf("X: X_init: mouse grabbing disabled\n");
   }
 
-  /* HACK!!! Don't konw is video_mode is already set */
+  /* start with some standard text mode */
   X_setmode(TEXT, co, li);
 
 #if CONFIG_X_SPEAKER
   register_speaker(display, X_speaker_on, X_speaker_off);
 #endif
 
-  return(0);                        
+  return 0;
 }
 
 
@@ -1309,13 +1334,8 @@ void X_close(void)
   destroy_ximage();
 
   vga_emu_done();
-  
-  if(vga256_cmap != 0) {
-      if (vga.mode_class == GRAPH)
-        XUninstallColormap(display, vga256_cmap);
 
-      XFreeColormap(display, vga256_cmap);
-    }
+  if(graphics_cmap) XFreeColormap(display, graphics_cmap);  
 
   XFreeGC(display, gc);
 
@@ -1408,7 +1428,7 @@ static int X_setmode(int mode_class, int text_width, int text_height)
       XSetWindowColormap(display, mainwindow, text_cmap);
 
       /* This is sometimes needed, when we return from graph to a saved
-       * text mode, else the screen would remain garbaged until we do manually
+       * text mode, else the screen would remain garbaged until we manually
        * do a resize/refresh.  --Hans
        *
        * Maybe, but then we should better do it always. -- Steffen
@@ -1455,7 +1475,7 @@ static int X_setmode(int mode_class, int text_width, int text_height)
 #endif
 
       if(!have_true_color) {
-        XSetWindowColormap(display, mainwindow, vga256_cmap);
+        XSetWindowColormap(display, mainwindow, graphics_cmap);
       }
 
       dac_bits = vga.dac.bits;
@@ -1519,8 +1539,6 @@ static int X_setmode(int mode_class, int text_width, int text_height)
         error("X: X_setmode: video_mode 0x%02x not supported on this screen\n", video_mode);
       }
 
-      InitDACTable();
-
       sh.width = w_x_res;
       sh.height = w_y_res;
       if(!(remap_obj.state & ROS_SCALE_ALL)) {
@@ -1579,10 +1597,6 @@ static int X_setmode(int mode_class, int text_width, int text_height)
 
   }
 
-  /* unconditionally update the palette */
-  DAC_dirty_all();
-  refresh_palette();
-
   if(X_map_mode != -1 && (X_map_mode == vga.mode || X_map_mode == vga.VESA_mode)) {
     XMapWindow(display, mainwindow);
     X_map_mode = -1;
@@ -1617,7 +1631,8 @@ static void X_modify_mode()
         X_printf("X: X_modify_mode: chain4 addressing turned %s\n", vga.mem.planes == 1 ? "on" : "off");
         remap_done(&remap_obj);
         remap_obj = tmp_ro;
-        InitDACTable();
+        /* Should not be not needed, as no colors have changed. -- sw */
+        /* update_all_colors(); */
       }
 
       dirty_all_video_pages();
@@ -2907,3 +2922,4 @@ struct video_system Video_X =
    X_update_screen,
    X_update_cursor
 };
+
