@@ -1,3 +1,9 @@
+/* 
+ * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /*
  * DANG_BEGIN_MODULE
  *
@@ -98,7 +104,7 @@
 
 
 #include <features.h>
-#if __GLIBC__ == 2 && __GLIBC_MINOR__ == 0
+#if GLIBC_VERSION_CODE == 2000
 #include <sigcontext.h>
 #endif
 #include <sys/mman.h>           /* root@sjoerd*/
@@ -605,11 +611,12 @@ int vga_emu_init(vgaemu_display_type *vedt)
   PRIV_SAVE_AREA
   int i;
   vga_mapping_type vmt = {0, 0, 0};
-#ifdef NEW_PORT_CODE
   emu_iodev_t io_device;
-#endif
   static unsigned char *lfb_base = NULL;
 
+  vga.mode = vga.VESA_mode = 0;
+  vga.mode_info = NULL;
+
   if(config.vgaemu_memsize)
     vga.mem.size = config.vgaemu_memsize << 10;
   else
@@ -669,13 +676,6 @@ int vga_emu_init(vgaemu_display_type *vedt)
 
   vga_emu_setup_mode_table();
 
-  /* initialize other parts */
-  DAC_init();
-  Attr_init();
-  Seq_init();
-  CRTC_init();
-
-#ifdef NEW_PORT_CODE
   /* register VGA ports */
   io_device.read_portb = VGA_emulate_inb;
   io_device.write_portb = VGA_emulate_outb;
@@ -709,10 +709,14 @@ int vga_emu_init(vgaemu_display_type *vedt)
   io_device.start_addr = CRTC_INDEX;
   io_device.end_addr = CRTC_DATA;
   port_register_handler(io_device, 0);
-#endif
 
   vbe_init(vedt);
 
+  /*
+   * Set some mode; this initializes the DAC, CRTC, etc. as well.
+   */
+  vga_emu_setmode(3, 80, 25);		/* initialize some mode */
+
   v_printf(
     "VGAEmu: vga_emu_init: memory = %u kbyte at 0x%x, lfb = 0x%x\n",
     vga.mem.size >> 10, (unsigned) vga.mem.base, vga.mem.map[VGAEMU_MAP_LFB_MODE].base_page << 12
@@ -1236,8 +1240,13 @@ int vga_emu_setmode(int mode, int width, int height)
   }
 
   vga.dac.bits = 6;
-  DAC_init();		/* Re-initialize the DAC */
 
+  DAC_init();		/* Re-initialize the DAC */
+  Attr_init();
+  Seq_init();
+  CRTC_init();
+
+  /* Should be moved to Seq_init()? -- sw */
   vga.seq.chain4 = 1;	/* ??? */
   vga.seq.map_mask = 1;
 
